@@ -14,12 +14,13 @@ use YAML::Any qw(LoadFile);
 use JSON qw();
 use Games::Lacuna::Client;
 use Games::Lacuna::Task::Utils qw(name_to_class);
+use Games::Lacuna::Task::Upgrade;
 
 our %LOCAL_CACHE;
 our $JSON = JSON->new->pretty(0)->utf8(1)->indent(0);
 our $API_KEY = '6ca1d525-bd4d-4bbb-ae85-b925ed3ea7b7';
 our $URI = 'https://us1.lacunaexpanse.com/';
-our @DB_TABLES = qw(star body cache empire);
+our @DB_TABLES = qw(star body cache empire meta);
 our @CONFIG_FILES = qw(lacuna config default);
 
 has 'client' => (
@@ -79,7 +80,6 @@ sub _build_config {
         require Games::Lacuna::Task::Setup;
         my $setup = Games::Lacuna::Task::Setup->new(
             configfile  => Path::Class::File->new($self->configdir,$CONFIG_FILES[0].'.yml')
-            
         );
         $global_config = $setup->run;
     }
@@ -154,7 +154,7 @@ sub _build_storage {
     # Connect database
     {
         no warnings 'once';
-        $storage = DBI->connect("dbi:SQLite:dbname=$storage_file","","")
+        $storage = DBI->connect("dbi:SQLite:dbname=$storage_file","","",{ sqlite_unicode => 1 })
             or $self->abort('Could not connect to database: %s',$DBI::errstr);
     }
     
@@ -197,6 +197,14 @@ sub _build_storage {
         
     }
     
+    # Upgrade storage
+    my $upgrade = Games::Lacuna::Task::Upgrade->new(
+        storage         => $storage,
+        loglevel        => $self->loglevel,
+        debug           => $self->debug,
+    );
+    $upgrade->run;
+    
     # Create distance function
     $storage->func( 'distance_func', 4, \&Games::Lacuna::Task::Utils::distance, "create_function" );
     
@@ -665,7 +673,8 @@ CREATE TABLE IF NOT EXISTS star (
   name TEXT NOT NULL,
   zone TEXT NOT NULL,
   last_checked INTEGER,
-  probed INTEGER 
+  is_probed INTEGER,
+  is_known INTEGER
 );
 
 CREATE TABLE IF NOT EXISTS body (
@@ -700,3 +709,9 @@ CREATE TABLE IF NOT EXISTS cache (
   valid_until INTEGER,
   checksum TEXT NOT NULL
 );
+
+
+CREATE TABLE IF NOT EXISTS meta ( 
+  key TEXT NOT NULL PRIMARY KEY, 
+  value TEXT NOT NULL
+);
\ No newline at end of file
