@@ -15,6 +15,9 @@ our %CARGO = (
 
 our $SCREEN_WIDTH = 78;
 
-our $MAX_MAP_QUERY = 30;
+our $MAX_MAP_QUERY = 30; # 30 x 30 units
+
+our $MAX_STAR_CACHE_AGE = 60*60*24*31*3; # Three months
+
 
 1;
\ No newline at end of file
