@@ -10,7 +10,7 @@ use Games::Lacuna::Task::Utils qw(normalize_name distance);
 use LWP::Simple;
 use Text::CSV;
 
-our $MAX_STAR_CACHE_AGE = 60*60*24*31*3; # Three months
+
 
 after 'BUILD' => sub {
     my ($self) = @_;
@@ -41,8 +41,8 @@ sub fetch_all_stars {
     $self->storage_do('CREATE TEMPORARY TABLE temporary_star (id INTEGER NOT NULL)');
     
     # Prepare sql statements
-    my $sth_check  = $self->storage_prepare('SELECT last_checked, probed FROM star WHERE id = ?');
-    my $sth_insert = $self->storage_prepare('INSERT INTO star (id,x,y,name,zone,last_checked,probed) VALUES (?,?,?,?,?,?,?)');
+    my $sth_check  = $self->storage_prepare('SELECT last_checked, is_probed, is_known FROM star WHERE id = ?');
+    my $sth_insert = $self->storage_prepare('INSERT INTO star (id,x,y,name,zone,last_checked,is_probed,is_known) VALUES (?,?,?,?,?,?,?,?)');
     my $sth_temp   = $self->storage_prepare('INSERT INTO temporary_star (id) VALUES (?)');
     
     # Parse star map
@@ -55,10 +55,10 @@ sub fetch_all_stars {
     my $count = 0;
     while( my $row = $csv->getline_hr( $fh ) ){
         $count++;
-        my ($last_checked,$probed);
+        my ($last_checked,$is_probed,$is_known);
         if ($check) {
             $sth_check->execute($row->{id});
-            ($last_checked,$probed) = $sth_check->fetchrow_array();
+            ($last_checked,$is_probed,$is_known) = $sth_check->fetchrow_array();
             $sth_check->finish();
         }
 
@@ -71,7 +71,8 @@ sub fetch_all_stars {
             $row->{name},
             $row->{zone},
             $last_checked,
-            $probed,
+            $is_probed,
+            $is_known
         );
         
         $self->log('debug',"Importing %i stars",$count)
@@ -88,7 +89,9 @@ sub fetch_all_stars {
 }
 
 sub _get_body_cache_for_star {
-    my ($self,$star_id,$star_data) = @_;
+    my ($self,$star_data) = @_;
+    
+    my $star_id = $star_data->{id}; 
     
     return
         unless defined $star_id;
@@ -139,7 +142,8 @@ sub _get_star_cache {
             star.name,
             star.zone,
             star.last_checked,
-            star.probed
+            star.is_probed,
+            star.is_known
         FROM star
         WHERE '.$query,
         {},
@@ -157,7 +161,7 @@ sub _inflate_star {
     
     # Build star data
     my $star_data = {
-        (map { $_ => $star_cache->{$_} } qw(id x y zone name probed last_checked)),
+        (map { $_ => $star_cache->{$_} } qw(id x y zone name is_probed is_known last_checked)),
         cache_ok    => 0,
     };
     
@@ -166,14 +170,14 @@ sub _inflate_star {
         unless defined $star_cache->{last_checked};
     
     # Get cache status
-    $star_data->{cache_ok} = ($star_cache->{last_checked} > (time() - $MAX_STAR_CACHE_AGE)) ? 1:0;
+    $star_data->{cache_ok} = ($star_cache->{last_checked} > (time() - $Games::Lacuna::Task::Constants::MAX_STAR_CACHE_AGE)) ? 1:0;
     
-    # We have no bodies and cache seems to be valid
+    # We have no bodies
     return $star_data
-        if $star_data->{cache_ok} == 1 && $star_data->{probed} == 0;
+        if defined $star_data->{is_known} && $star_data->{is_known} == 0;
     
     # Get Bodies from cache
-    my @bodies = $self->_get_body_cache_for_star($star_data->{id},$star_data);
+    my @bodies = $self->_get_body_cache_for_star($star_data);
     
     # Bodies ok
     if (scalar @bodies) {
@@ -250,7 +254,8 @@ sub _get_body_cache {
             star.name AS star_name,
             star.zone AS zone,
             star.last_checked,
-            star.probed,
+            star.is_probed,
+            star.is_known,
             empire.id AS empire_id,
             empire.name AS empire_name,
             empire.alignment AS empire_alignment,
@@ -299,20 +304,24 @@ sub get_body_by_xy {
         && $x =~ m/^-?\d+$/
         && $y =~ m/^-?\d+$/;
     
-    $self->_get_body_cache('body.x = ? AND body.y = ?',$x,$y);
+    return $self->_get_body_cache('body.x = ? AND body.y = ?',$x,$y);
     
-    my ($star_data) = $self->list_stars(
-        x       => $x,
-        y       => $y,
-        limit   => 1,
-        distance=> 1,
-    );
-    
-    foreach my $body_data (@{$star_data->{bodies}}) {
-        return $body_data
-            if $body_data->{x} == $x
-            && $body_data->{y} == $y;
-    }
+#    my ($star_data) = $self->list_stars(
+#        x       => $x,
+#        y       => $y,
+#        limit   => 1,
+#        distance=> 1,
+#    );
+#    
+#    return
+#        unless defined $star_data
+#        && defined $star_data->{bodies};
+#    
+#    foreach my $body_data (@{$star_data->{bodies}}) {
+#        return $body_data
+#            if $body_data->{x} == $x
+#            && $body_data->{y} == $y;
+#    }
     
     return;
 }
@@ -359,12 +368,29 @@ sub _get_star_api {
     my $star_list = $self->_get_area_api_by_xy($min_x,$min_y,$max_x,$max_y);
     
     # Find star in list
+    my $star_data;
     foreach my $element (@{$star_list}) {
-        return $element
-            if $element->{id} == $star_id;
+        if ($element->{id} == $star_id) {
+            $star_data = $element;
+            last;
+        }
     }
     
-    return;
+    # Get bodies from cache, even if system is not probed
+    if (! defined $star_data->{bodies}
+        && $star_data->{is_known} == 1) {
+        
+        my @bodies = $self->_get_body_cache_for_star($star_data);
+        if (scalar @bodies) {
+            $star_data->{bodies} = \@bodies;
+        } else {
+            $self->log('warn','Inconsitent cache state for star %i',$star_data->{id});
+            $self->storage_do('UPDATE star SET is_known = ?, is_probed = 0 WHERE id = ?',0,0,$star_data->{id});
+        }
+    }
+    
+    
+    return $star_data;
 }
 
 
@@ -437,14 +463,23 @@ sub set_star_cache {
     return
         unless defined $star_id;
     
+    delete $star_data->{bodies}
+        if (defined $star_data->{bodies} && scalar @{$star_data->{bodies}} == 0);
     $star_data->{last_checked} ||= time();
     $star_data->{cache_ok} //= 1;
-    $star_data->{probed} //= (defined $star_data->{bodies} && scalar @{$star_data->{bodies}} ? 1:0);
+    $star_data->{is_probed} //= (defined $star_data->{bodies} ? 1:0);
+    $star_data->{is_known} //= 1
+        if $star_data->{is_probed};
+    
+    unless (defined $star_data->{is_known}) {
+        ($star_data->{is_known}) = $self->client->storage->selectrow_array('SELECT COUNT(1) FROM body WHERE star = ?',{},$star_id);
+    }
     
     # Update star cache
     $self->storage_do(
-        'UPDATE star SET probed = ?, last_checked = ?, name = ? WHERE id = ?',
-        $star_data->{probed},
+        'UPDATE star SET is_probed = ?, is_known = ?, last_checked = ?, name = ? WHERE id = ?',
+        $star_data->{is_probed},
+        $star_data->{is_known},
         $star_data->{last_checked},
         $star_data->{name},
         $star_id
@@ -516,7 +551,7 @@ sub search_stars_callback {
     my @sql_where;
     my @sql_params;
     my @sql_extra;
-    my @sql_fields = qw(star.id star.x star.y star.name star.zone star.last_checked star.probed);
+    my @sql_fields = qw(star.id star.x star.y star.name star.zone star.last_checked star.is_probed star.is_known);
     
     # Order by distance
     if (defined $params{distance}
@@ -536,9 +571,14 @@ sub search_stars_callback {
         push(@sql_extra," ORDER BY distance ".($params{distance} ? 'ASC':'DESC'));
     }
     # Only probed/unprobed or unknown
-    if (defined $params{probed}) {
-        push(@sql_where,'(star.last_checked < ? OR star.probed = ? OR star.probed IS NULL)');
-        push(@sql_params,(time - $MAX_STAR_CACHE_AGE),$params{probed});
+    if (defined $params{is_probed}) {
+        push(@sql_where,'(star.last_checked < ? OR star.is_probed = ? OR star.is_probed IS NULL)');
+        push(@sql_params,(time - $Games::Lacuna::Task::Constants::MAX_STAR_CACHE_AGE),$params{is_probed});
+    }
+    # Only known/unknown 
+    if (defined $params{is_known}) {
+        push(@sql_where,'(star.is_known = ? OR star.is_known IS NULL)');
+        push(@sql_params,$params{is_known});
     }
     # Zone
     if (defined $params{zone}) {
@@ -573,7 +613,7 @@ sub search_stars_callback {
         # Inflate star data
         my $star_data;
         if (defined $star_cache->{last_checked} 
-            && $star_cache->{last_checked} > (time - $MAX_STAR_CACHE_AGE)) {
+            && $star_cache->{last_checked} > (time - $Games::Lacuna::Task::Constants::MAX_STAR_CACHE_AGE)) {
             $star_data = $self->_inflate_star($star_cache);
         } else {
             $star_data = $self->_get_star_api($star_cache->{id},$star_cache->{x},$star_cache->{y});
@@ -581,7 +621,11 @@ sub search_stars_callback {
         
         # Check definitve probed status
         next
-            if (defined $params{probed} && $star_data->{probed} != $params{probed});
+            if (defined $params{is_probed} && $star_data->{is_probed} != $params{is_probed});
+        
+        # Check definitve known status
+        next
+            if (defined $params{is_known} && $star_data->{is_known} != $params{is_known});
         
         # Set distance
         $star_data->{distance} = $star_cache->{distance}
@@ -705,7 +749,9 @@ Valid search options are
 
 =over
 
-=item * probed (0 = unprobed, 1 = probed)
+=item * is_probed (0 = unprobed, 1 = probed)
+
+=item * is_known (0 = body data not available, 1 = body data available)
 
 =item * max_distance
 
